import sys
import os
import subprocess

external_tools_path=['','/usr/bin/','/usr/local/bin/']
internal_tools_path=['../']+external_tools_path

class Tools:
  demux_command=None
  mux_command=None
  codecs_command=None
  kateenc_command=None
  katedec_command=None

  def __init__(self,on_start=None,on_end=None):
    self.on_start=on_start
    self.on_end=on_end
    self.find_tools()

  def probe_command(self,command,options,magic):
    cmdline=command+' '+options
    try:
      popen=subprocess.Popen(cmdline,stdin=None,stderr=subprocess.PIPE,stdout=subprocess.PIPE,universal_newlines=True,shell=True)
    except OSError as e:
      return None
    if not popen.stdout:
      return None
    if popen.stdin:
      popen.stdin.close()
    if popen.stderr:
      popen.stderr.close()
    line=popen.stdout.readline()
    while line:
      if magic in line:
        popen.stdout.close()
        return command
      line=popen.stdout.readline()
    popen.stdout.close()
    return None

  def probe_command_in(self,commands,options,magic,paths):
    for command in commands:
      for path in paths:
        found=self.probe_command(path+command,options,magic)
        if found!=None:
          return found
    return None

  def check(self):
    msg=''
    if self.demux_command==None:
      msg+='Failed to find the oggz-rip demuxing tool\n'

    if self.mux_command==None:
      msg+='Failed to find the oggz-merge muxing tool\n'

    if self.kateenc_command==None:
      msg+='Failed to find kateenc\n'

    if self.katedec_command==None:
      msg+='Failed to find katedec\n'

    # oggz-known-codecs is ok not to be found

    if msg!='':
      raise Exception(msg)

  def find_tools(self):
    self.mux_command=self.probe_command_in(['oggz-merge','oggzmerge'],'-v','merge version',external_tools_path)
    self.demux_command=self.probe_command_in(['oggz-rip','oggzrip'],'-v','rip version',external_tools_path)
    self.codecs_command=self.probe_command_in(['oggz-known-codecs'],'-v','oggz-known-codecs version',external_tools_path)
    self.kateenc_command=self.probe_command_in(['kateenc'],'-V','Kate reference encoder - libkate',internal_tools_path)
    self.katedec_command=self.probe_command_in(['katedec'],'-V','Kate reference decoder - libkate',internal_tools_path)
    self.check()

  def run(self,params,stdin):
    if self.on_start!=None: self.on_start()
    try:
      try:
        popen=subprocess.Popen(params,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE,universal_newlines=True)
        if stdin==None:
          (stdoutdata,stderrdata)=popen.communicate(stdin)
        else:
          (stdoutdata,stderrdata)=popen.communicate(stdin.encode('utf8'))
        ret=popen.returncode
        # oggz tools can return 0 when they fail, so do not test ret
        msg=stderrdata.encode('utf8')
        if msg!=None and msg!='':
          raise Exception(msg)
      except:
        raise
    finally:
      if self.on_end!=None: self.on_end()
  
  def run_demux(self,params,stdin=None):
    if self.demux_command==None:
      raise Exception('No demux command found')
    self.run([self.demux_command]+params,stdin)

  def run_mux(self,params,stdin=None):
    if self.mux_command==None:
      raise Exception('No mux command found')
    self.run([self.mux_command]+params,stdin)

  def run_katedec(self,params,stdin=None):
    if self.katedec_command==None:
      raise Exception('katedec was not found')
    self.run([self.katedec_command]+params,stdin)

  def run_kateenc(self,params,stdin=None):
    if self.kateenc_command==None:
      raise Exception('kateenc was not found')
    self.run([self.kateenc_command]+params,stdin)

if __name__=='__main__':
  tools=Tools();




import sys
import os
import wx
from .ui_editor import UIEditor
from .ui_options import UIOptions

from .constants import *
from .tools import Tools
from .demuxer import Demuxer
from .muxer import Muxer
from .finder import FindKateStreams
from .options import Options

base_width=480
base_height=240
serial_width=int(base_width/4)
language_width=int(base_width/4)
category_width=int(base_width/2)
padding=8
button_width=int((base_width-padding*2)/3)
button_height=32
list_height=96

class UIMain(wx.Frame):
  def __init__(self):
    wx.Frame.__init__(self,None,wx.ID_ANY,kdj_name_version)

    self.options=Options(wx.StandardPaths.Get().GetUserConfigDir())

    try:
      self.tools=Tools(wx.BeginBusyCursor,wx.EndBusyCursor)
    except Exception as e:
      wx.MessageBox('Failed to find necessary tools:\n'+str(e),'Error',style=wx.OK|wx.CENTRE|wx.ICON_ERROR)
      sys.exit(1)

    self.filename=None
    self.demuxer=None

    vbox=wx.BoxSizer(wx.VERTICAL)
    self.SetSizer(vbox)

    self.action_button=self.AddActionButton()
    vbox.Add(self.action_button,0,wx.EXPAND|wx.ALL,border=padding)

    self.list=self.AddList()
    vbox.Add(self.list,1,wx.EXPAND|wx.ALL,border=padding)

    bbox=wx.BoxSizer(wx.HORIZONTAL)
    vbox.Add(bbox,0,wx.EXPAND|wx.ALL,border=padding)
    self.options_button=self.AddOptionsButton()
    bbox.Add(self.options_button,0,wx.EXPAND)
    bbox.Add((padding,0),1,wx.EXPAND)
    self.help_button=self.AddHelpButton()
    bbox.Add(self.help_button,0,wx.EXPAND)
    self.quit_button=self.AddQuitButton()
    bbox.Add((padding,0),1,wx.EXPAND)
    bbox.Add(self.quit_button,0,wx.EXPAND)

    self.SetMinSize((base_width+2*padding,base_height))
    self.Fit()
    self.Show(True)

  def SetupActionButton(self,button):
    if self.filename==None:
      button.SetLabel('Load Ogg stream')
      button.Bind(wx.EVT_BUTTON,self.OnLoadButton,button)
    elif self.demuxer!=None:
      button.SetLabel('Remux file from parts')
      button.Bind(wx.EVT_BUTTON,self.OnMuxButton,button)
    else:
      button.SetLabel('Demux file')
      button.Bind(wx.EVT_BUTTON,self.OnDemuxButton,button)

  def AddButton(self,text):
    button=wx.Button(self,wx.ID_ANY,text)
    return button

  def AddLoadButton(self):
    button=self.AddButton('Load file')
    button.Bind(wx.EVT_BUTTON,self.OnLoadButton,button)
    return button

  def AddActionButton(self):
    button=self.AddButton('')
    self.SetupActionButton(button)
    return button

  def AddOptionsButton(self):
    button=wx.Button(self,wx.ID_ANY,'Options',size=(button_width,button_height))
    button.Bind(wx.EVT_BUTTON,self.OnOptionsButton,button)
    return button

  def AddHelpButton(self):
    button=wx.Button(self,wx.ID_HELP,'Help',size=(button_width,button_height))
    button.Bind(wx.EVT_BUTTON,self.OnHelpButton,button)
    return button

  def AddQuitButton(self):
    button=wx.Button(self,wx.ID_EXIT,'Quit',size=(button_width,button_height))
    button.Bind(wx.EVT_BUTTON,self.OnQuitButton,button)
    return button

  def OnListSizeChanged(self,event):
    self.Refresh()
    event.Skip()

  def Edit(self,list_idx=-1):
    wx.BeginBusyCursor()
    dlg=UIEditor(self,self.tools,format=self.options.format)
    try:
      kate_streams=FindKateStreams(self.demuxer.GetDirectory())
      if list_idx>=0:
        idx=self.list.GetItemData(list_idx)
        dlg.addStream(kate_streams[str(idx)]['filename'])
      else:
        for idx2 in kate_streams:
          dlg.addStream(kate_streams[idx2]['filename'])
      dlg.ShowModal()
    except Exception as e:
      print('Exception: %s' % str(e))
    dlg.Destroy()
    wx.EndBusyCursor()

  def OnListDoubleClicked(self,event):
    idx=self.list.GetFirstSelected()
    if idx>=0:
      self.Edit(idx)
    event.Skip()

  def AddList(self):
    list=wx.ListCtrl(self,style=wx.LC_REPORT)
    list.ClearAll()
    list.InsertColumn(0,'Serial')
    list.InsertColumn(1,'Language')
    list.InsertColumn(2,'Category')
    list.SetColumnWidth(0,serial_width)
    list.SetColumnWidth(1,language_width)
    list.SetColumnWidth(2,category_width)
    list.Bind(wx.EVT_SIZE,self.OnListSizeChanged)
    list.Bind(wx.EVT_LEFT_DCLICK,self.OnListDoubleClicked)
    return list

  def CheckAndContinue(self):
    if self.filename!=None and self.demuxer!=None:
      ret=wx.MessageBox('Loaded file not remuxed - Quit anyway ?','Warning',parent=self,style=wx.YES|wx.NO_DEFAULT|wx.CENTRE|wx.ICON_QUESTION)
      if ret!=wx.YES:
        return False
    return True

  def SetFilename(self,dir,file):
    filename=os.path.join(dir,file)
    # TODO: some *quick* oggness checks
    self.filename=filename
    self.SetupActionButton(self.action_button)

  def OnLoadButton(self,event):
    if not self.CheckAndContinue():
      return
    extensions=['ogg','oga','ogv','ogx']
    spec='Ogg streams|'
    for ext in extensions:
      spec+='*.'+ext+';'
      spec+='*.'+ext.capitalize()+';'
      spec+='*.'+ext.upper()+';'
    spec+='|All files|*;*.*'
    loader=wx.FileDialog(self,message='Load Ogg stream',wildcard=spec)
    ret=loader.ShowModal()
    if ret==wx.ID_OK:
      filename=loader.GetFilename()
      dirname=loader.GetDirectory()
      self.SetFilename(dirname,filename)

  def FillKateList(self,demuxer):
    streams=FindKateStreams(demuxer.GetDirectory())
    for idx in streams:
      stream=streams[idx]
      self.list.Append(['','',''])
      list_idx=self.list.GetItemCount()-1
      self.list.SetStringItem(list_idx,0,stream['serial'])
      self.list.SetStringItem(list_idx,1,stream['language'])
      self.list.SetStringItem(list_idx,2,stream['category'])
      self.list.SetItemData(list_idx,int(idx))
    return self.list.GetItemCount()>0

  def OnDemuxButton(self,event):
    try:
      demuxer=Demuxer(self.tools,self.filename,self.options.format)
    except Exception as e:
      wx.MessageBox('Failed to demux file:\n'+str(e),'Error',parent=self,style=wx.OK|wx.CENTRE|wx.ICON_ERROR)
      return

    if not self.FillKateList(demuxer):
      wx.MessageBox('No Kate streams found, or not an Ogg stream','Error',parent=self,style=wx.OK|wx.CENTRE|wx.ICON_ERROR)
      self.filename=None
      self.SetupActionButton(self.action_button)
      return

    self.demuxer=demuxer
    self.SetupActionButton(self.action_button)
    path=os.path.abspath(self.demuxer.GetDirectory())
    wx.MessageBox(
      'Demuxed files are in '+path+'.\n'+
        'Double click on any entry to edit it.\n'+
        'You may also edit them with your favorite text editor.',
      'Edit files',
      parent=self,
      style=wx.OK|wx.CENTRE
    )

  def RemoveTemporaryFiles(self,directory):
    streams=FindKateStreams(directory)
    for idx in streams:
      stream=streams[idx]
      filename=stream['filename']
      os.unlink(filename)
      if os.access(filename+'.ogg',os.F_OK):
        os.unlink(filename+'.ogg')
    os.unlink(os.path.join(directory,'misc.ogg'))
    os.rmdir(directory)

  def OnMuxButton(self,event):
    directory=self.demuxer.GetDirectory()
    try:
      if self.options.save_as_copy:
        root,ext=os.path.splitext(self.filename)
        remuxed_name=root+'.remuxed'+ext
      else:
        remuxed_name=self.filename
      muxer=Muxer(self.tools,remuxed_name,directory,'kate')
      while self.list.GetItemCount()>0: self.list.DeleteItem(0)
      self.demuxer=None
      self.SetupActionButton(self.action_button)
      try:
        self.RemoveTemporaryFiles(directory)
      except:
        wx.MessageBox('Failed to remove all temporary files from\n%s' % directory,'Error',parent=self,style=wx.OK|wx.CENTRE|wx.ICON_ERROR)
    except Exception as e:
      wx.MessageBox('Failed to remux file:\n'+str(e),'Error',parent=self,style=wx.OK|wx.CENTRE|wx.ICON_ERROR)

  def OnOptionsButton(self,event):
    dlg=UIOptions(self,self.options)
#    dlg.CenterOnScreen()
    dlg.ShowModal()
    dlg.Destroy()

  def OnHelpButton(self,event):
    wx.MessageBox(
      kdj_name+' is a remuxing program that allows extracting and decoding Kate tracks from an Ogg stream, '+
      'and recreating that Ogg stream after the Kate streams have been altered.\n'+
      '\n'+
      kdj_name+' requires both the Kate tools (kateenc and katedec) and the oggz tools.'+
      '\n\n'+
      'Click \'Load Ogg stream\' to load an Ogg file from disk. All types of Ogg streams are supported '+
      '(eg, audio, video, etc).\n'+
      '\n'+
      'Then, click \'Demux file\' to extract Kate streams, and decode them to a temporary place on disk. '+
      'You may now edit the decoded Kate streams, either by double clicking on the one you wish to change, '+
      'or with your text editor of choice.\n'+
      '\n'+
      'When done, click \'Remux file from parts\' to reconstruct the Ogg stream with the modified Kate streams. '+
      'If you are notified of any error (eg, syntax errors in the modified Kate streams), then go back and '+
      'correct any mistakes, and click \'Remux file from parts\' until no errors are reported.\n'
      '\n'+
      'You may now play the rebuilt Ogg stream in your audio/video player, and repeat the demux/remux two step '+
      'process until you are happy with the resulting stream.\n'+
      '',
      kdj_name+' help',
      parent=self,
      style=wx.OK|wx.CENTRE
    )

  def OnQuitButton(self,event):
    if not self.CheckAndContinue():
      return
    if self.demuxer!=None and self.options.remove_temporary_files:
      directory=self.demuxer.GetDirectory()
      try:
        self.RemoveTemporaryFiles(directory)
      except:
        wx.MessageBox('Failed to remove all temporary files from\n%s' % directory,'Error',parent=self,style=wx.OK|wx.CENTRE|wx.ICON_ERROR)
    sys.exit(0)




import sys
import os

from .tools import Tools
from .finder import FindKateStreams

class Muxer:
  def __init__(self,tools,filename,directory,type):
    self.directory=directory
    self.Mux(tools,filename,type)

  def EncodeKateStream(self,tools,kate_stream,type):
    filename=kate_stream['filename']
    params=[]
    params+=['-t',type]
    params+=['-o',filename+'.ogg']
    params+=[filename]
    params+=['-l',kate_stream['language']]
    params+=['-c',kate_stream['category']]
    params+=['-s',kate_stream['serial']]
    tools.run_kateenc(params)

  def Mux(self,tools,filename,type):
    kate_streams=FindKateStreams(self.directory)

    params=[]
    params+=['-o',filename,os.path.join(self.directory,'misc.ogg')]
    for idx in kate_streams:
      kate_stream=kate_streams[idx]
      self.EncodeKateStream(tools,kate_stream,type)
      params+=[kate_stream['filename']+'.ogg']
    tools.run_mux(params)

if __name__=='__main__':
  tools=Tools()
  dir='katedj-tmp-extract-8ELIgg'
  if len(sys.argv)>1:
    dir=sys.argv[1];
  muxer=Muxer(tools,'newdemo.ogg',dir)



import sys
import os
import tempfile
  

from .tools import Tools

class Demuxer:
  def __init__(self,tools,filename,type):
    if not self.CreateDirectory():
      raise Exception('Failed to create directory')
    self.Demux(tools,filename,type)

  def GetDirectory(self):
    return self.directory

  def CreateDirectory(self):
    try:
      self.directory=tempfile.mkdtemp(dir='.',prefix='katedj-tmp-extract-')
    except OSError as e:
      return False
    return True

  def GetCodecs(self,tools):
    cmdline=tools.codecs_command
    if cmdline!=None:
      try:
        popen=subprocess.Popen(cmdline,stdin=None,stderr=subprocess.PIPE,stdout=subprocess.PIPE,universal_newlines=True,shell=True)
        if popen.stdout:
          list=[]
          line=popen.stdout.readline()
          while line:
            line=line.split('\n')[0]
            if line and 'Kate' not in line:
              list.append(line)
            line=popen.stdout.readline()
          popen.stdout.close()
          return list
      except:
        pass
    return ['theora','vorbis','dirac','speex','flac','cmml']
    
  def DemuxMisc(self,tools,filename):
    params=[]
    params+=['-o',os.path.join(self.directory,'misc.ogg')]
    for codec in self.GetCodecs(tools):
      params+=['-c',codec]
    params+=[filename]
    tools.run_demux(params)

  def DemuxKate(self,tools,filename,type):
    params=[]
    params+=['-o',os.path.join(self.directory,'kate.%l.%c.%i.%s.kate')]
    params+=['-t',type]
    params+=[filename]
    tools.run_katedec(params)

  def Demux(self,tools,filename,type):
    self.DemuxMisc(tools,filename)
    self.DemuxKate(tools,filename,type)

if __name__=='__main__':
  tools=Tools()
  file='../../built-streams/demo.ogg'
  if len(sys.argv)>1:
    file=sys.argv[1];
  demuxer=Demuxer(tools,file)



import sys
import os

from .tools import Tools

class Tester:
  def __init__(self,tools,contents,format):
    self.Test(tools,contents,format)

  def Test(self,tools,contents,format):
    params=[]
    params+=['-t',format]
    params+=['-o','-']
    params+=['-l','x-test','-c','test']
    tools.run_kateenc(params,stdin=contents)




import sys
import os

def FindKateStreams(directory):
    dict={}
    for root, dirs, files in os.walk(directory):
      for file in files:
        if '.kate'==file[-5:]:
          filename=os.path.join(root,file)
          fragments=filename.split('.') # kate.%l.%c.%i.%s.kate
          if fragments and len(fragments)>=6:
            language=fragments[-5]
            category=fragments[-4]
            index=fragments[-3]
            serial=fragments[-2]
            dict[index]={'filename':filename,'language':language,'category':category,'serial':serial}
          else:
            print('Bad filename format: '+filename)

    return dict




import sys
import wx
from wx.py.editor import EditorNotebook
from wx.py.buffer import Buffer
from wx.py.editor import Editor
from .constants import *
from .tester import Tester

class UIEditor(wx.Dialog):
  def __init__(self,parent,tools,filename=None,format='kate'):
    pre=wx.PreDialog()
    pre.Create(parent,wx.ID_ANY,title='Editing Kate streams',pos=(100,100),size=(600,400),style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
    self.PostCreate(pre)

    self.tools=tools
    self.format=format

    box=wx.BoxSizer(wx.VERTICAL)
    self.SetSizer(box)

    self.notebook=EditorNotebook(self)
    box.Add(self.notebook,1,wx.EXPAND)
    self.notebook.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED,self.OnPageChanged)
    if filename!=None:
      self.addStream(filemame)

    buttons=wx.BoxSizer(wx.HORIZONTAL)
    box.Add(buttons)

    test=wx.Button(self,wx.ID_ANY,'Test')
    test.Bind(wx.EVT_BUTTON,self.OnTest)
    buttons.Add(test)

    save=wx.Button(self,wx.ID_SAVE,'Save')
    save.Bind(wx.EVT_BUTTON,self.OnSave)
    buttons.Add(save)

#    save_all=wx.Button(self,wx.ID_SAVE,'Save all')
#    save_all.Bind(wx.EVT_BUTTON,self.OnSaveAll)
#    buttons.Add(save_all)

    help=wx.Button(self,wx.ID_HELP,'Help')
    help.Bind(wx.EVT_BUTTON,self.OnHelp)
    buttons.Add(help)

    quit=wx.Button(self,wx.ID_EXIT,'Quit')
    quit.Bind(wx.EVT_BUTTON,self.OnQuit)
    buttons.Add(quit)

    self.buffers=[]

    self.Layout()
    self.Show(True)

  def addStream(self,filename):
     buffer=Buffer()
     panel=wx.Panel(parent=self.notebook,id=-1)
     panel.Bind(wx.EVT_ERASE_BACKGROUND,lambda x: x)
     editor=Editor(parent=panel)

     panel.editor=editor
     panel.buffer=buffer
     panel.filename=filename

     sizer=wx.BoxSizer(wx.VERTICAL)
     sizer.Add(editor.window,1,wx.EXPAND)
     panel.SetSizer(sizer)
     panel.SetAutoLayout(True)
     sizer.Layout()
     buffer.addEditor(editor)
     buffer.open(filename)
     self.notebook.AddPage(page=panel,text=buffer.name,select=True)
     if (self.notebook.GetPageCount()==1):
       self.OnPageSelected(0)
     self.buffers.append(buffer)
     editor.setFocus()

  def OnPageSelected(self,idx):
    self.current_panel=self.notebook.GetPage(idx)

  def OnPageChanged(self,event):
    new=event.GetSelection()
    self.OnPageSelected(new)
    event.Skip()

  def OnTest(self,event):
    try:
      tester=Tester(self.tools,self.current_panel.editor.getText(),self.format)
      wx.MessageBox('No errors found','Success',parent=self,style=wx.OK)
    except Exception as e:
      wx.MessageBox('Error:\n'+str(e),'Test failed',parent=self,style=wx.OK|wx.ICON_ERROR)

  def OnSave(self,event):
    self.current_panel.buffer.save()
    event.Skip()

  def OnSaveAll(self,event):
    for buffer in self.buffers:
      buffer.save()
    event.Skip()

  def getNumChanged(self):
    changed=0
    for buffer in self.buffers:
      if buffer.hasChanged():
        changed+=1
    return changed

  def OnHelp(self,event):
    wx.MessageBox(
      'Edit the decoded Kate stream, and save when done. '+
      '\n'+
      'Make liberal use of the Test button, which will tell you if any error is found in the stream. '+
      '\n'+
      'Note that there is no need to save before testing.'+
      '\n\n'+
      '',
      kdj_name+' help',
      parent=self,
      style=wx.OK|wx.CENTRE
    )

  def OnQuit(self,event):
    changed=self.getNumChanged()
    quit=True
    if changed>0:
      dlg=wx.MessageDialog(self,'There are unsaved changes. Quit anyway ?','Unsaved changes')
      if dlg.ShowModal()!=wx.ID_OK:
        quit=False
      dlg.Destroy()
    if quit:
      self.Close(True)




import os

class Options:
  def __init__(self,path):
    self.rc=os.path.join(path,'.KateDJrc')
    #print 'options: config file is '+self.rc
    try:
      self.SetDefaults()
      self.Load()
    except:
      pass

  def SetDefaults(self):
    self.save_as_copy=True
    self.remove_temporary_files=False
    self.format='kate'

  def Load(self):
    f=open(self.rc,'Ur')
    if not f:
      return
    try:
      try:
        line=f.readline()
        while line:
          line=self.Frob(line)
          self.Parse(line)
          line=f.readline()
      except:
        pass
    finally:
      f.close()
    
  def Save(self):
    f=open(self.rc,'w')
    f.write('save_as_copy='+self.Bool2String(self.save_as_copy)+'\n')
    f.write('remove_temporary_files='+self.Bool2String(self.remove_temporary_files)+'\n')
    f.write('format='+self.format.lower()+'\n')
    f.close()

  def Bool2String(self,val):
    if val:
      return 'true'
    else:
      return 'false'

  def String2Bool(self,val):
    val=val.lower()
    if val=='true' or val=='1' or val=='yes' or val=='y':
      return True
    else:
      return False

  def String2Format(self,val):
    val=val.lower()
    if val=='kate':
      return 'kate'
    if val=='srt':
      return 'srt'
    if val=='lrc':
      return 'lrc'
    return 'kate'

  def Frob(self,line):
    line=line.split('#')[0]
    line=line.split('\n')[0]
    return line

  def Parse(self,line):
    var,val=line.split('=')[0:2]
    if var.lower()=='save_as_copy': self.save_as_copy=self.String2Bool(val)
    elif var.lower()=='remove_temporary_files': self.remove_temporary_files=self.String2Bool(val)
    elif var.lower()=='format': self.format=self.String2Format(val)
    #else: print 'unknown variable: '+line




import sys
import wx
from .constants import *

base_width=480
option_height=24
help_height=32+16*4
button_height=32
hpadding=16
vpadding=8

class UIOptions(wx.Dialog):
  def __init__(self,parent,options):
    pre=wx.PreDialog()
    pre.Create(parent,wx.ID_ANY,title=kdj_name+' options')
    self.PostCreate(pre)

    self.options=options

    self.y=vpadding
    self.opt_save_as_copy=self.AddCheckBox(
      'Save remuxed file as a copy of the original Ogg file',
      options.save_as_copy,
      'If enabled, remuxed files will be saved on a different file, so the source '+
      'file is left untouched. The new file will be named similarly to the old file, '+
      'but with \'.remuxed\' inserted before the extension.'
    )
    self.y+=vpadding
    self.opt_remove=self.AddCheckBox(
      'Remove all temporary files on exit',
      options.remove_temporary_files,
      'If enabled, any temporary files extracted from an Ogg stream will be, '+
      'removed, even if the file was not remuxed. Any changes that may have '+
      'been made to the Kate streams in these temporary files will be lost.'
    )
    self.y+=vpadding
    self.opt_format=self.AddList(
      'Decode Kate streams as',
      ['Kate', 'SRT', 'LRC'],
      options.format,
      'Selects the text format to use when decoding Kate streams.\n'+
      'Typically, text based movie subtitles are best decoded as SRT, and other '+
      'streams as Kate.'
    )
    self.y+=vpadding

    help_pos=(hpadding,self.y)
    help_size=(base_width,help_height)
    help_inset=(16,24)
    wx.StaticBox(self,label='More info',pos=help_pos,size=help_size)
    help_pos=(help_pos[0]+help_inset[0],help_pos[1]+help_inset[1])
    self.help_size=(help_size[0]-2*help_inset[0],help_size[1]-2*help_inset[1])
    self.help_text=wx.StaticText(self,wx.ID_ANY,'',pos=help_pos,size=help_size)
    self.y+=help_height
    self.y+=vpadding

    self.accept_button=wx.Button(self,wx.ID_OK,'OK',(hpadding,self.y),((base_width-hpadding)/2,button_height))
    self.cancel_button=wx.Button(self,wx.ID_CANCEL,'Cancel',(hpadding*2+base_width/2,self.y),((base_width-hpadding)/2,button_height))
    self.accept_button.Bind(wx.EVT_BUTTON,self.OnOK)
    self.y+=button_height

    self.y+=vpadding
    self.SetSize((base_width+hpadding*2,self.y))
    self.Show(True)

  def SetHelpText(self,text):
    self.help_text.SetLabel(text)

  def SetHelpTextAndSkip(self,text,ev):
    self.help_text.SetLabel(text)
    self.help_text.Wrap(self.help_size[0])
    ev.Skip()

  def AddButton(self,id,label):
    button=wx.Button(self,id=id) #,label=label)
    return button

  def AddCheckBox(self,label,on,help_text):
    checkbox=wx.CheckBox(self,label=label,pos=(hpadding,self.y),size=(base_width,option_height))
    checkbox.Bind(wx.EVT_ENTER_WINDOW,lambda ev: self.SetHelpTextAndSkip(help_text,ev))
    checkbox.Bind(wx.EVT_LEAVE_WINDOW,lambda ev: self.SetHelpTextAndSkip('',ev))
    checkbox.SetValue(on)
    self.y+=option_height
    return checkbox

  def AddList(self,text,choices,default,help_text):
    box=wx.BoxSizer(wx.HORIZONTAL)
    box.SetDimension(hpadding,self.y,base_width,option_height)

    label=wx.StaticText(self,wx.ID_ANY,text)
    box.Add(label,flag=wx.ALIGN_CENTER_VERTICAL)
    label.Bind(wx.EVT_ENTER_WINDOW,lambda ev: self.SetHelpTextAndSkip(help_text,ev))
    label.Bind(wx.EVT_LEAVE_WINDOW,lambda ev: self.SetHelpTextAndSkip('',ev))

    box.AddSpacer((8,0))

    list=wx.Choice(self, -1, (80,-1),choices=choices)
    list.SetStringSelection(default)
    box.Add(list,flag=wx.ALIGN_CENTER_VERTICAL)
    list.Bind(wx.EVT_ENTER_WINDOW,lambda ev: self.SetHelpTextAndSkip(help_text,ev))
    list.Bind(wx.EVT_LEAVE_WINDOW,lambda ev: self.SetHelpTextAndSkip('',ev))

    box.Layout()
    self.y+=option_height
    return list

  def OnOK(self,event):
    self.options.save_as_copy=self.opt_save_as_copy.IsChecked()
    self.options.remove_temporary_files=self.opt_remove.IsChecked()
    self.options.format=self.opt_format.GetStringSelection().lower()
    self.options.Save()
    event.Skip()

